﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using ObycejnyGraf;
using MG.DatoveStruktury.LinearniMnoziny;
using MG.DatoveStruktury.Tabulky;
using MG.OperacniVyzkum.TeorieGrafu;
using MG.DatoveStruktury.Grafy.SmisenaReprezentace;

namespace Graf
{
  /// <summary>
  /// Hlavní formulář ukázkové aplikace.
  /// </summary>
  public partial class FrmHlavni : Form
  {
    #region Datové složky
    GVrchol v1, v2;
    #endregion


    #region Eventy
    private void lbSeznamVrcholu_DoubleClick(object sender, EventArgs e)
    {
      GVrchol gv = lbSeznamVrcholu.SelectedItem as GVrchol;
      if (gv == null)
        return;
      Komponenta.UpravVrchol(gv);
      lbSeznamVrcholu.Items.RemoveAt(lbSeznamVrcholu.SelectedIndex);
      lbSeznamVrcholu.Items.Add(gv);
    }

    private void lbSeznamHran_DoubleClick(object sender, EventArgs e)
    {
      GHrana gh = lbSeznamHran.SelectedItem as GHrana;
      if (gh == null)
        return;
      Komponenta.UpravHranu(gh);
      lbSeznamHran.Items.RemoveAt(lbSeznamHran.SelectedIndex);
      lbSeznamHran.Items.Add(gh);
    }

    private void btnPridatVrchol_Click(object sender, EventArgs e)
    {
      try
      {
        GVrchol gv = new GVrchol(new SouradniceVrcholu(0, 0));
        gv.BarvaOhraniceni = Komponenta.VnejsiBarvaVrcholu;
        gv.BarvaVyplne = Komponenta.VnitrniBarvaVrcholu;
        gv.Velikost = Komponenta.VelikostVrcholu;
        gv.ZobrazitText = Komponenta.ZobrazitTextVrcholu;
        gv.FontTextu = Komponenta.FontVrcholu;
        Komponenta.UpravVrchol(gv);
        Komponenta.Graf.VlozVrchol(gv);
        lbSeznamVrcholu.Items.Add(gv);
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message, "Chyba aplikace", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnPridatHranu_Click(object sender, EventArgs e)
    {
      using (FrmVyberDvouVrcholu frm = new FrmVyberDvouVrcholu(Komponenta.Graf, "Vyberte incidenční vrcholy hrany.",
        "1. incidenční vrchol", "2. incidenční vrchol"))
      {
        if (frm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
          v1 = frm.PrvniIncidencniVrchol;
          v2 = frm.DruhyIncidencniVrchol;

          try
          {
            GHrana gh = new GHrana(v1, v2);
            gh.Barva = Komponenta.BarvaHrany;
            gh.BarvaTextu = Komponenta.BarvaTextuHrany;
            gh.FontTextu = Komponenta.FontHrany;
            gh.Ohodnoceni = Komponenta.OhodnoceniHrany;
            gh.Tloustka = Komponenta.TloustkaHrany;
            gh.ZobrazitText = Komponenta.ZobrazitTextHrany;
            Komponenta.UpravHranu(gh);
            Komponenta.Graf.VlozHranu(gh, gh.Klic);
            lbSeznamHran.Items.Add(gh);
          }
          catch (Exception ex)
          {
            MessageBox.Show(ex.Message, "Chyba aplikace", MessageBoxButtons.OK, MessageBoxIcon.Error);
          }
        }
      }
    }

    private void btnUpravitVrchol_Click(object sender, EventArgs e)
    {
      lbSeznamVrcholu_DoubleClick(this, null);
    }

    private void btnUpravitHranu_Click(object sender, EventArgs e)
    {
      lbSeznamHran_DoubleClick(this, null);
    }

    private void btnOdstranitVrchol_Click(object sender, EventArgs e)
    {
      GVrchol gv = lbSeznamVrcholu.SelectedItem as GVrchol;
      if (gv == null)
        return;

      if (gv.Stupen != 0)
      {
        MessageBox.Show("Stupeň vrcholu není 0 (vrchol není bez incidenční hrany).", "Chyba aplikace",
          MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }

      GVrchol gv2 = Komponenta.Graf.OdstranVrchol(gv.Klic);

      lbSeznamVrcholu.Items.RemoveAt(lbSeznamVrcholu.SelectedIndex);
    }

    private void btnOdstranitHranu_Click(object sender, EventArgs e)
    {
      GHrana gh = lbSeznamHran.SelectedItem as GHrana;
      if (gh == null)
        return;

      GHrana gh2 = Komponenta.Graf.OdstranHranu(gh.Klic);
      lbSeznamHran.Items.RemoveAt(lbSeznamHran.SelectedIndex);
    }

    private void tsmiBinarneUlozit_Click(object sender, EventArgs e)
    {
      sfdlgUlozitGraf.Filter = "Bin Files (*.bin)|*.bin";
      if (sfdlgUlozitGraf.ShowDialog() == DialogResult.OK)
        Komponenta.UlozGrafBinarne(sfdlgUlozitGraf.FileName);
    }

    private void tsmiBinarneNacist_Click(object sender, EventArgs e)
    {
      ofdlgNacistGraf.Filter = "Bin Files (*.bin)|*.bin";
      if (ofdlgNacistGraf.ShowDialog() == DialogResult.OK)
      {
        try
        {
          Komponenta.NactiGrafBinarne(ofdlgNacistGraf.FileName);

          lbSeznamHran.Items.Clear();
          lbSeznamVrcholu.Items.Clear();

          foreach (GVrchol item in Komponenta.Graf)
          {
            lbSeznamVrcholu.Items.Add(item);
          }

          foreach (IPrvekTabulky<GHrana, double> item in Komponenta.Graf.SeznamHran)
          {
            lbSeznamHran.Items.Add(item.Data);
          }

          lblPocHran.Text = Komponenta.Graf.PocetHran.ToString();
          lblPocVrch.Text = Komponenta.Graf.PocetVrcholu.ToString();
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message, "Chyba aplikace", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
      }
    }

    private void Komponenta_MouseMove(object sender, MouseEventArgs e)
    {
      lblX.Text = Komponenta.SouradniceKurzoru.X.ToString();
      lblY.Text = Komponenta.SouradniceKurzoru.Y.ToString();
    }

    private void Komponenta_KeyDown(object sender, KeyEventArgs e)
    {
      if (e.KeyCode == Keys.CapsLock)
        if (Komponenta.RezimPrace == TypRezmimuPrace.PraceSVrcholy)
          Komponenta.RezimPrace = TypRezmimuPrace.PraceSHranami;
        else
          Komponenta.RezimPrace = TypRezmimuPrace.PraceSVrcholy;
      
    }

    private void Komponenta_ZmenaStrukturyGrafu(object sender, ObycejnyGraf.ZmenaStrukturyGrafuEventArgs e)
    {
      lblPocHran.Text = Komponenta.Graf.PocetHran.ToString();
      lblPocVrch.Text = Komponenta.Graf.PocetVrcholu.ToString();

      switch (e.TypZmeny)
      {
        case TypZmenyStrukturyGrafu.VlozeniVrcholu:
        Komponenta.UpravVrchol((GVrchol)e.Objekt);
        lbSeznamVrcholu.Items.Add(e.Objekt);
        break;
        case TypZmenyStrukturyGrafu.EditaceVrcholu:
        lbSeznamVrcholu.Items.RemoveAt(lbSeznamVrcholu.Items.IndexOf(e.Objekt));
        lbSeznamVrcholu.Items.Add(e.Objekt);
        break;
        case TypZmenyStrukturyGrafu.OdstraneniVrcholu:
        lbSeznamVrcholu.Items.RemoveAt(lbSeznamVrcholu.Items.IndexOf(e.Objekt));
        break;
        case TypZmenyStrukturyGrafu.VlozeniHrany:
        Komponenta.UpravHranu((GHrana)e.Objekt);
        lbSeznamHran.Items.Add(e.Objekt);
        break;
        case TypZmenyStrukturyGrafu.EditaceHrany:
        lbSeznamHran.Items.RemoveAt(lbSeznamHran.Items.IndexOf(e.Objekt));
        lbSeznamHran.Items.Add(e.Objekt);
        break;
        case TypZmenyStrukturyGrafu.OdstraneniHrany:
        lbSeznamHran.Items.RemoveAt(lbSeznamHran.Items.IndexOf(e.Objekt));
        break;
      }
    }

    private void Komponenta_Cancel(object sender, ObycejnyGraf.ZmenaStrukturyGrafuCancelEventArgs ec)
    {
      ec.Cancel =
      (
        MessageBox.Show
        (
          "Opravdu chcete odstranit " +
          (
            (ec.Objekt.GetType().ToString() == "ObycejnyGraf.GVrchol") ?
              ("vrchol " + ((GVrchol)ec.Objekt).Nazev)
            :
              ("hranu " + ((GHrana)ec.Objekt).Nazev)
          ) + "?",
          "Potvrzení odstranění",
          MessageBoxButtons.OKCancel,
          MessageBoxIcon.Warning
        ) == System.Windows.Forms.DialogResult.OK
      ) ?
          false
        :
          true;
    }

    private void btnKonec_Click(object sender, EventArgs e)
    {
      Close();
    }

    /// <summary>
    /// Nastane při pokusu o ukončení apliakce.
    /// </summary>
    /// <param name="e">Doplňkové informace.</param>
    protected override void OnFormClosing(FormClosingEventArgs e)
    {
      base.OnFormClosing(e);

      if (e.Cancel || e.CloseReason == CloseReason.WindowsShutDown)
        return;

      if (MessageBox.Show("Opravdu chcete ukončit aplikaci?", "Konec programu",
         MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question,
         MessageBoxDefaultButton.Button2) != 
            System.Windows.Forms.DialogResult.Yes)
      {
        e.Cancel = true;
      }
    }

    private void tsmiCesta_Click(object sender, EventArgs e)
    {
      using (FrmVyberDvouVrcholu frm = new FrmVyberDvouVrcholu(Komponenta.Graf, "Vyberte Počátečný a koncový vrchol cesty.",
        "Počátečný vrchol cesty", "Koncový vrchol cesty"))
      {
        if (frm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
        {
          v1 = frm.PrvniIncidencniVrchol;
          v2 = frm.DruhyIncidencniVrchol;

          try
          {
            string typVypoctu = "";
            AdsObousmernySeznam<IPrvekGrafu> seznam = null;
            if ((sender as ToolStripMenuItem) == tsmiNejkratsiCesta)
            {
              typVypoctu = "Délka cesty: ";
              MessageBox.Show("Dojde k pokusu o vypočítání nejkratší cesty. Graf musí být obyčejný," +
                "souvislý na hledané cestě a ohodnocení hran musí být z intervalu (0 ; nekonečno) kromě.",
              "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
              seznam = CestyNaGrafech<GVrchol, GHrana, double, double, double>.NejkratsiCesta(Komponenta.Graf,
                v1, v2, x => x);
            }
            else if ((sender as ToolStripMenuItem) == tsmiNejspolehlivejsiCesta)
            {
              typVypoctu = "Spolehlivost cesty: ";
              MessageBox.Show("Dojde k pokusu o vypočítání nejspolehlivější cesty. Graf musí být obyčejný," +
                "souvislý na hledané cestě a ohodnocení hran musí být z intervalu <0 ; 1> včetně.",
              "Informace", MessageBoxButtons.OK, MessageBoxIcon.Information);
              seznam = CestyNaGrafech<GVrchol, GHrana, double, double, double>.NejspolehlivejsiCesta(Komponenta.Graf,
              v1, v2, x => x);
            }

            string cesta = "Směr cesty: ";
            string hodnotaCesty = typVypoctu + (seznam.ZpristupniPosledni() as GVrchol).Tag.ToString();
            foreach (IPrvekGrafu prvek in seznam)
            {
              cesta += prvek.ToString();
              if (prvek is GVrchol)
              {
                (prvek as GVrchol).BarvaVyplne = Color.Red;
              }
              else
              {
                (prvek as GHrana).Barva = Color.Red;
              }
            }
            MessageBox.Show(cesta + ". " + hodnotaCesty);
          }
          catch (Exception ex)
          {
            MessageBox.Show(ex.Message);
          }
        }
      }
    }
    #endregion


    #region Konstruktor
    /// <summary>
    /// Bezparametrický konstruktor. Inicializuje komponentu.
    /// </summary>
    public FrmHlavni()
    {
      InitializeComponent();
      lbSeznamVrcholu.DisplayMember = "Nazev";
      lbSeznamHran.DisplayMember = "Nazev";
    }
    #endregion
  }
}