﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ObycejnyGraf;

namespace Graf
{
  /// <summary>
  /// Formulář sloužící k výběru dvou vrcholů.
  /// </summary>
  public partial class FrmVyberDvouVrcholu : Form
  {
    /// <summary>
    /// Parametrický konstruktor.
    /// </summary>
    /// <param name="graf">Graf, jehož vrcholy se vybírají.</param>
    /// <param name="popisekHlavni">Popisek formuláře.</param>
    /// <param name="popisekV1">Popisek prvního vrcholu.</param>
    /// <param name="popisekV2">Popisek druhého vrcholu.</param>
    public FrmVyberDvouVrcholu(GGraf graf, string popisekHlavni, string popisekV1, string popisekV2)
    {
      InitializeComponent();
      lbl_Popis.Text = popisekHlavni;
      lbl_PopisV1.Text = popisekV1;
      lbl_PopisV2.Text = popisekV2;

      foreach (GVrchol v in graf)
      {
        cbV1.Items.Add(v);
        cbV2.Items.Add(v);
      }

      cbV1.DisplayMember = "Nazev";
      cbV2.DisplayMember = "Nazev";
    }

    #region Vlastnosti
    /// <summary>
    /// První vybraný incidenční vrchol.
    /// </summary>
    public GVrchol PrvniIncidencniVrchol { get; private set; }

    /// <summary>
    /// Druhý vybraný incidenční vrchol.
    /// </summary>
    public GVrchol DruhyIncidencniVrchol { get; private set; }
    #endregion

    #region Eventy
    /// <summary>
    /// Nastane při pokusu o zavření formuláře. Provede kontrolu vstupu.
    /// </summary>
    /// <param name="e">Doplňující informace.</param>
    protected override void OnFormClosing(FormClosingEventArgs e)
    {
      base.OnFormClosing(e);

      if (DialogResult == System.Windows.Forms.DialogResult.Cancel || e.Cancel)
        return;

      if (cbV1.SelectedIndex == -1 || cbV2.SelectedIndex == -1)
      {
        MessageBox.Show("Některý z vrcholů není vybraný", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
        e.Cancel = true;
      }
      else
        if (ReferenceEquals(cbV1.SelectedItem, cbV2.SelectedItem))
        {
          MessageBox.Show("Graf nepodporuje smyčky", "Chyba aplikace", MessageBoxButtons.OK, MessageBoxIcon.Error);
          e.Cancel = true;
        }
    }

    /// <summary>
    /// Nastane při zavření dialogu. Zpřístupní označené vrcholy.
    /// </summary>
    /// <param name="e">Doplňující informace</param>
    protected override void OnFormClosed(FormClosedEventArgs e)
    {
      base.OnFormClosed(e);

      if (DialogResult != System.Windows.Forms.DialogResult.Cancel)
      {
        PrvniIncidencniVrchol = cbV1.SelectedItem as GVrchol;
        DruhyIncidencniVrchol = cbV2.SelectedItem as GVrchol;
      }
    }
    #endregion
  }
}
