﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;

namespace ObycejnyGraf
{
  /// <summary>
  /// Přepravka uchovávající hodnoty nastavení komponenty <see cref="ObycejnyGraf"/>.
  /// </summary>
  [Serializable]
  sealed public class NastaveniPrepravka
  {
    #region Datové složky
    private Color barvaPozadi;
    private Color barvaOsy;
    private Color vnejsiBarvaVrcholu;
    private Color vnitrniBarvaVrcholu;
    private Color barvaHrany;
    private Color barvaTextuVrcholu;
    private Color barvaTextuHrany;
    private Color barvaAktivnihoPrvku;
    private int velikostVrcholu;
    private Font fontVrcholu;
    private bool zobrazitTextVrcholu;
    private int tloustkaHrany;
    private double ohodnoceniHrany;
    private Font fontHrany;
    private bool zobrazitTextHrany;
    #endregion


    #region  Vlastnosti
    /// <summary>
    /// Barva pozadí komponenty.
    /// </summary>
    public Color BarvaPozadi
    {
      get { return barvaPozadi; }
    }
    /// <summary>
    /// Barva osy komponenty.
    /// </summary>
    public Color BarvaOsy
    {
      get { return barvaOsy; }
    }
    /// <summary>
    /// Vnější barva vrcholu grafu.
    /// </summary>
    public Color VnejsiBarvaVrcholu
    {
      get { return vnejsiBarvaVrcholu; }
    }
    /// <summary>
    /// Vnitřní barva vrcholu grafu.
    /// </summary>
    public Color VnitrniBarvaVrcholu
    {
      get { return vnitrniBarvaVrcholu; }
    }
    /// <summary>
    /// Barva hrany.
    /// </summary>
    public Color BarvaHrany
    {
      get { return barvaHrany; }
    }
    /// <summary>
    /// Barva textu vrcholu.
    /// </summary>
    public Color BarvaTextuVrcholu
    {
      get { return barvaTextuVrcholu; }
    }
    /// <summary>
    /// Barva textu hrany.
    /// </summary>
    public Color BarvaTextuHrany
    {
      get { return barvaTextuHrany; }
    }
    /// <summary>
    /// Barva aktivního prvku komponenty.
    /// </summary>
    public Color BarvaAktivnihoPrvku
    {
      get { return barvaAktivnihoPrvku; }
    }
    /// <summary>
    /// Velikost kružnice vrcholu.
    /// </summary>
    public int VelikostVrcholu
    {
      get { return velikostVrcholu; }
    }
    /// <summary>
    /// Font vrcholu.
    /// </summary>
    public Font FontVrcholu
    {
      get { return fontVrcholu; }
    }
    /// <summary>
    /// Zobrazení textu vrcholu.
    /// </summary>
    public bool ZobrazitTextVrcholu
    {
      get { return zobrazitTextVrcholu; }
    }
    /// <summary>
    /// Tloušťka úsečky hrany.
    /// </summary>
    public int TloustkaHrany
    {
      get { return tloustkaHrany; }
    }
    /// <summary>
    /// Ohodnocení hrany.
    /// </summary>
    public double OhodnoceniHrany
    {
      get { return ohodnoceniHrany; }
    }
    /// <summary>
    /// Font hrany.
    /// </summary>
    public Font FontHrany
    {
      get { return fontHrany; }
    }
    /// <summary>
    /// Zobrazení textu hrany.
    /// </summary>
    public bool ZobrazitTextHrany
    {
      get { return zobrazitTextHrany; }
    }
    #endregion


    #region Konstruktor
    /// <summary>
    /// Parametrický konstruktor.
    /// </summary>
    /// <param name="barvaPozadi"></param>
    /// <param name="barvaOsy"></param>
    /// <param name="vnejsiBarvaVrcholu"></param>
    /// <param name="vnitrniBarvaVrcholu"></param>
    /// <param name="barvaHrany"></param>
    /// <param name="barvaTextuVrcholu"></param>
    /// <param name="barvaTextuHrany"></param>
    /// <param name="barvaAktualnihoPrvku"></param>
    /// <param name="velikostVrcholu"></param>
    /// <param name="fontVrcholu"></param>
    /// <param name="zobrazitTextVrcholu"></param>
    /// <param name="tloustkaHrany"></param>
    /// <param name="ohodnoceniHrany"></param>
    /// <param name="fontHrany"></param>
    /// <param name="zobrazitTextHrany"></param>
    public NastaveniPrepravka(Color barvaPozadi, Color barvaOsy, Color vnejsiBarvaVrcholu, Color vnitrniBarvaVrcholu,
      Color barvaHrany, Color barvaTextuVrcholu, Color barvaTextuHrany, Color barvaAktualnihoPrvku,
      int velikostVrcholu, Font fontVrcholu, bool zobrazitTextVrcholu,
      int tloustkaHrany, double ohodnoceniHrany, Font fontHrany, bool zobrazitTextHrany)
    {
      this.barvaPozadi = barvaPozadi;
      this.barvaOsy = barvaOsy;
      this.vnejsiBarvaVrcholu = vnejsiBarvaVrcholu;
      this.vnitrniBarvaVrcholu = vnitrniBarvaVrcholu;
      this.barvaAktivnihoPrvku = barvaAktualnihoPrvku;
      this.barvaHrany = barvaHrany;
      this.barvaTextuVrcholu = barvaTextuVrcholu;
      this.barvaTextuHrany = barvaTextuHrany;

      this.velikostVrcholu = velikostVrcholu;
      this.fontVrcholu = fontVrcholu;
      this.zobrazitTextVrcholu = zobrazitTextVrcholu;

      this.ohodnoceniHrany = ohodnoceniHrany;
      this.tloustkaHrany = tloustkaHrany;
      this.fontHrany = fontHrany;
      this.zobrazitTextHrany = zobrazitTextHrany;
    }
    #endregion
  }
}
