﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ObycejnyGraf
{
  /// <summary>
  /// Formulářové dialogové okno pro nastavení komponenty grafu.
  /// </summary>
  partial class FrmNastaveni : Form
  {
    #region Datové složky
    private bool posun;
    private Point soucasnaLokace = new Point();
    #endregion

    #region Vlastnosti
    /// <summary>
    /// Informace o provedení resetování implicitního nastavení na prvcích komponenty.
    /// </summary>
    public bool Reset { get; private set; }

    /// <summary>
    /// Přepravka s nastavením.
    /// </summary>
    public NastaveniPrepravka Nastaveni { get; set; }
    #endregion

    #region Eventy
    private void btnBarvaPozadi_Click(object sender, EventArgs e)
    {
      cdlgBarvaPozadi.ShowDialog();
      btnBarvaPozadi.BackColor = cdlgBarvaPozadi.Color;
    }

    private void btnBarvaOsy_Click(object sender, EventArgs e)
    {
      cdlgBarvaOsy.ShowDialog();
      btnBarvaOsy.BackColor = cdlgBarvaOsy.Color;
    }

    private void btnVnejsiBarvaVrcholu_Click(object sender, EventArgs e)
    {
      cdlgVnejsiBarvaVrcholu.ShowDialog();
      btnVnejsiBarvaVrcholu.BackColor = cdlgVnejsiBarvaVrcholu.Color;
    }

    private void btnVnitrniBarvaVrcholu_Click(object sender, EventArgs e)
    {
      cdlgVnitrniBarvaVrcholu.ShowDialog();
      btnVnitrniBarvaVrcholu.BackColor = cdlgVnitrniBarvaVrcholu.Color;
    }

    private void btnBarvaHrany_Click(object sender, EventArgs e)
    {
      cdlgBarvaHrany.ShowDialog();
      btnBarvaHrany.BackColor = cdlgBarvaHrany.Color;
    }

    private void btnBarvaAKtualnihoPrvku_Click(object sender, EventArgs e)
    {
      cdlgBarvaAktualnihoPrvku.ShowDialog();
      btnBarvaAKtualnihoPrvku.BackColor = cdlgBarvaAktualnihoPrvku.Color;
    }

    private void btnBarvaTextuVrcholu_Click(object sender, EventArgs e)
    {
      cdlgBarvaTextuVrcholu.ShowDialog();
      btnBarvaTextuVrcholu.BackColor = cdlgBarvaTextuVrcholu.Color;
    }

    private void btnBarvaTextuHrany_Click(object sender, EventArgs e)
    {
      cdlgBarvaTextuHrany.ShowDialog();
      btnBarvaTextuHrany.BackColor = cdlgBarvaTextuHrany.Color;
    }

    private void btnFontVrcholu_Click(object sender, EventArgs e)
    {
      fdFontVrcholu.ShowDialog();
    }

    private void btnFontHrany_Click(object sender, EventArgs e)
    {
      fdFontHrany.ShowDialog();
    }

    private void frmNastaveni_MouseDown(object sender, MouseEventArgs e)
    {
      posun = true;
      soucasnaLokace.X = e.X;
      soucasnaLokace.Y = e.Y;
    }


    private void frmNastaveni_MouseMove(object sender, MouseEventArgs e)
    {
      if (posun)
      {
        Location = new Point(Location.X + e.X - soucasnaLokace.X, Location.Y + e.Y - soucasnaLokace.Y);
        soucasnaLokace.X = e.X;
        soucasnaLokace.Y = e.Y;
      }
    }

    private void frmNastaveni_MouseUp(object sender, MouseEventArgs e)
    {
      posun = false;
    }

    private void btnReset_Click(object sender, EventArgs e)
    {
      Reset = true;
      MessageBox.Show("Prvky grafu budou zresetovány dle současné konfigurace (kromě ohodnocení hran)." + 
        "Tato volba bude platná po potvrzení hlavního dialogového okna");
    }

    protected override void OnFormClosed(FormClosedEventArgs e)
    {
      base.OnFormClosed(e);

      if (DialogResult != System.Windows.Forms.DialogResult.Cancel)
      {
        Nastaveni = new NastaveniPrepravka(btnBarvaPozadi.BackColor, btnBarvaOsy.BackColor,
            btnVnejsiBarvaVrcholu.BackColor, btnVnitrniBarvaVrcholu.BackColor, btnBarvaHrany.BackColor,
            btnBarvaTextuVrcholu.BackColor, btnBarvaTextuHrany.BackColor, btnBarvaAKtualnihoPrvku.BackColor,
            (int)nudVelikostVrcholu.Value, fdFontVrcholu.Font, cbZobrazitTextVrcholu.Checked,
            (int)nudTloustka.Value, (double)nudOhodnoceni.Value, fdFontHrany.Font, cbZobrazitTextHrany.Checked);
      }
    }
    #endregion

    #region Konstruktor
    /// <summary>
    /// Parametrický konstruktor. Inicializuje formulář.
    /// </summary>
    /// <param name="p">Přepravka se současnými hodnotami nastavení komponenty.</param>
    internal FrmNastaveni(NastaveniPrepravka p)
    {
      InitializeComponent();

      Reset = false;

      btnBarvaAKtualnihoPrvku.BackColor = p.BarvaAktivnihoPrvku;
      btnBarvaHrany.BackColor = p.BarvaHrany;
      btnBarvaOsy.BackColor = p.BarvaOsy;
      btnBarvaPozadi.BackColor = p.BarvaPozadi;
      btnBarvaTextuHrany.BackColor = p.BarvaTextuHrany;
      btnBarvaTextuVrcholu.BackColor = p.BarvaTextuVrcholu;
      btnVnejsiBarvaVrcholu.BackColor = p.VnejsiBarvaVrcholu;
      btnVnitrniBarvaVrcholu.BackColor = p.VnitrniBarvaVrcholu;

      nudVelikostVrcholu.Value = p.VelikostVrcholu;
      fdFontVrcholu.Font = p.FontVrcholu;
      cbZobrazitTextVrcholu.Checked = p.ZobrazitTextVrcholu;

      nudOhodnoceni.Value = (int)p.OhodnoceniHrany;
      nudTloustka.Value = p.TloustkaHrany;
      fdFontHrany.Font = p.FontHrany;
      cbZobrazitTextHrany.Checked = p.ZobrazitTextHrany;
    }
    #endregion
  }
}
