﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MG.DatoveStruktury.LinearniMnoziny
{
  /// <summary>
  /// Abstraktní datová struktura zásobník na lineárním seznamu. 
  /// <remarks>Složitost všech operací je O(1).</remarks>
  /// </summary>
  /// <typeparam name="T">Referenční Datový typ, který obsahují prvky zásobníku.</typeparam>
  public class AdsZasobnik<T> : IEnumerable<T>
  {
    #region Datové složky
    private AdsObousmernySeznam<T> seznam = new AdsObousmernySeznam<T>();
    #endregion


    #region Vlastnosti
    /// <summary>
    /// <c>True</c> je-li zásobník prázdný (0 prvků v zás.), <c>False</c> jinak.
    /// </summary>
    public bool JePrazdny
    {
      get { return seznam.JePrazdny; }
    }
    #endregion


    #region Metody
    /// <summary>
    /// Vloží prvek do zásobníku.
    /// </summary>
    /// <param name="data">Data, která má prvek obsahovat.</param>
    public void VlozPrvek(T data)
    {
      seznam.VlozPrvni(data);
    }

    /// <summary>
    /// Odebere prvek ze zásobníku.
    /// </summary>
    /// <returns>Data prvku typu <typeparamref name="T"/>
    /// nebo Default(<typeparamref name="T"/>), je-li zásobník prázdný.</returns>
    public T odeberPrvek()
    {
      return seznam.OdeberPrvni();
    }

    /// <summary>
    /// Zpřístupní prvek ze zásobníku.
    /// </summary>
    /// <returns>Data prvku typu <typeparamref name="T"/>
    /// nebo Default(<typeparamref name="T"/>), je-li zásobník prázdný.</returns>
    public T ZpristupniPrvek()
    {
      return seznam.ZpristupniPrvni();
    }

    /// <summary>
    /// Vytvoří enumerátor.
    /// </summary>
    /// <returns>Enumerátor zásobníku.</returns>
    public IEnumerator<T> GetEnumerator()
    {
      T prvek = default(T);
      prvek = seznam.ZpristupniPrvni();
      while (prvek != null)
      {
        yield return prvek;
        prvek = seznam.ZpristupniNaslednika();
      }
    }

    System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
    {
      return GetEnumerator();
    } 
    #endregion


    #region Konstruktor
    /// <summary>
    /// Bezparametrický konstruktor.
    /// </summary>
    public AdsZasobnik() { }

    /// <summary>
    /// Parametrický konstruktor. Vloží prvek s danými daty do zásobníku.
    /// </summary>
    /// <param name="data">Data, která má prvek obsahovat.</param>
    public AdsZasobnik(T data)
    {
      VlozPrvek(data);
    }
    #endregion
  }
}
