﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MG.Matematika.Geometrie2D
{
  /// <summary>
  /// Obsahuje souřadnice <see cref="X"/> a <see cref="Y"/> daného typu <c><typeparamref name="T"/></c>. 
  /// </summary>
  /// <typeparam name="T">Typ souřadnic bodu. Typ musí implementovat rozhraní <see cref="IComparable"/>.</typeparam>
  [Serializable]
  public class Bod<T> where T : IComparable<T>
  {
    #region Vlastnosti
    /// <summary>
    /// X-ová souřadnice.
    /// </summary>
    public virtual T X { get; set; }

    /// <summary>
    /// Y-ová souřadnice.
    /// </summary>
    public virtual T Y { get; set; }
    #endregion


    #region Konstruktor
    /// <summary>
    /// Bezparametrický konstruktor.
    /// </summary>
    public Bod() { }

    /// <summary>
    /// Inicializční konstruktor. Inicializuje vlastnosti <see cref="X"/> a <see cref="Y"/>.
    /// </summary>
    /// <param name="x">Souřadncie X daného bodu.</param>
    /// <param name="y">Souřadnice Y daného bodu.</param>
    public Bod(T x, T y)
    {
      this.X= x;
      this.Y= y;
    }
    #endregion
  }
}
