using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MG.Matematika.Geometrie2D
{
  /// <summary>
  /// Tda poskytuje geometrick funcke pro prci v 2D souadnm systmu.
  /// </summary>
  public sealed class FunkceGeometricke
  {
    #region Metody
    /// <summary>
    /// Zjist, zda se bod nachz na sece dan svmi krajnmi body vzhledem k dan pesnosti.
    /// </summary>
    /// <param name="krajniBodUsecky1">Prvn krajn bod seky.</param>
    /// <param name="krajniBodUsecky2">Druh krajn bod seky.</param>
    /// <param name="bod">Zkouman bod.</param>
    /// <param name="presnost">Pesnost vpotu v desetinch slech.</param>
    /// <returns><c>True</c> pokud se bod nachz na sece, <c>False</c> jinak.</returns>
    public static bool JeBodNaUsecce(Bod<double> krajniBodUsecky1, Bod<double> krajniBodUsecky2, Bod<double> bod, int presnost)
    {
      if (bod.X <= ((krajniBodUsecky1.X > krajniBodUsecky2.X) ? krajniBodUsecky1.X : krajniBodUsecky2.X))
      {
        if (bod.X >= ((krajniBodUsecky1.X < krajniBodUsecky2.X) ? krajniBodUsecky1.X : krajniBodUsecky2.X))
        {
          if (bod.Y <= ((krajniBodUsecky1.Y > krajniBodUsecky2.Y) ? krajniBodUsecky1.Y : krajniBodUsecky2.Y))
          {
            if (bod.Y >= ((krajniBodUsecky1.Y < krajniBodUsecky2.Y) ? krajniBodUsecky1.Y : krajniBodUsecky2.Y))
            {
              if (krajniBodUsecky2.X - krajniBodUsecky1.X != 0) // k != nekoneno
              {
                double k = (krajniBodUsecky2.Y - krajniBodUsecky1.Y) / (krajniBodUsecky2.X - krajniBodUsecky1.X);
                double q = krajniBodUsecky1.Y - k * krajniBodUsecky1.X;
                if (presnost >= 0)
                {
                  // debug mezivypocet
                  double y = Math.Round(bod.Y, presnost);
                  double x = Math.Round(k * bod.X + q, presnost);
                  if (y == x)
                    return true;
                  else
                    return false;
                }
                else
                {
                  if (bod.Y == k * bod.X + q)
                    return true;
                  else
                    return false;
                }
              }
              else
                return true;
            }
          }
        }
      }

      return false;
    }

    /// <summary>
    /// Zjist, zda se bod nachz na sece dan svmi krajnmi body (pesnost vpotu na rozsah dat. typu <see cref="double"/>).
    /// </summary>
    /// <param name="krajniBodUsecky1">Prvn krajn bod seky.</param>
    /// <param name="krajniBodUsecky2">Druh krajn bod seky.</param>
    /// <param name="bod">Zkouman bod.</param>
    /// <returns><c>True</c> pokud se bod nachz na sece, <c>False</c> jinak.</returns>
    public static bool JeBodNaUsecce(Bod<double> krajniBodUsecky1, Bod<double> krajniBodUsecky2, Bod<double> bod)
    {
      return JeBodNaUsecce(krajniBodUsecky1, krajniBodUsecky2, bod, -1);
    }

    /// <summary>
    /// Vrt novou instanci Bodu se souadnicemi stedu dan seky.
    /// </summary>
    /// <param name="krajniBodUsecky1">Prvn krajn bod seky.</param>
    /// <param name="krajniBodUsecky2">Druh krajn bod seky.</param>
    /// <returns>Bod se souadnicemi stedu seky.</returns>
    public static Bod<double> StredUsecky(Bod<double> krajniBodUsecky1, Bod<double> krajniBodUsecky2)
    {
      Bod<double> s = new Bod<double>();
      s.X = ((krajniBodUsecky1.X > krajniBodUsecky2.X) ? krajniBodUsecky2.X : krajniBodUsecky1.X) +
        Math.Abs((krajniBodUsecky2.X - krajniBodUsecky1.X) / 2);
      s.Y = ((krajniBodUsecky1.Y > krajniBodUsecky2.Y) ? krajniBodUsecky2.Y : krajniBodUsecky1.Y) +
        Math.Abs((krajniBodUsecky2.Y - krajniBodUsecky1.Y) / 2);
      return s;
    }

    /// <summary>
    /// Vypote dlku seky.
    /// </summary>
    /// <param name="krajniBodUsecky1">Prvn krajn bod seky.</param>
    /// <param name="krajniBodUsecky2">Druh krajn bod seky.</param>
    /// <returns>Dlku dan seky.</returns>
    public static double DelkaUsecky(Bod<double> krajniBodUsecky1, Bod<double> krajniBodUsecky2) 
    {
      return (Math.Sqrt(Math.Pow(krajniBodUsecky2.X - krajniBodUsecky1.X,2) +
        Math.Pow(krajniBodUsecky2.Y - krajniBodUsecky1.Y,2)));
    }
    #endregion


    #region Konstruktor
    private FunkceGeometricke() { }
    #endregion
  }
}